module GHC.Version where

import Prelude -- See Note [Why do we import Prelude here?]

cProjectGitCommitId   :: String
cProjectGitCommitId   = "29204b1c4f52ea34d84da33593052ee839293bf2"

cProjectVersion       :: String
cProjectVersion       = "8.10.2"

cProjectVersionInt    :: String
cProjectVersionInt    = "810"

cProjectPatchLevel    :: String
cProjectPatchLevel    = "2"

cProjectPatchLevel1   :: String
cProjectPatchLevel1   = "2"

cProjectPatchLevel2   :: String
cProjectPatchLevel2   = ""
