    cONTROL_GROUP_CONST_291,
    sTD_HDR_SIZE,
    pROF_HDR_SIZE,
    bLOCK_SIZE,
    bLOCKS_PER_MBLOCK,
    tICKY_BIN_COUNT,
    oFFSET_StgRegTable_rR1,
    oFFSET_StgRegTable_rR2,
    oFFSET_StgRegTable_rR3,
    oFFSET_StgRegTable_rR4,
    oFFSET_StgRegTable_rR5,
    oFFSET_StgRegTable_rR6,
    oFFSET_StgRegTable_rR7,
    oFFSET_StgRegTable_rR8,
    oFFSET_StgRegTable_rR9,
    oFFSET_StgRegTable_rR10,
    oFFSET_StgRegTable_rF1,
    oFFSET_StgRegTable_rF2,
    oFFSET_StgRegTable_rF3,
    oFFSET_StgRegTable_rF4,
    oFFSET_StgRegTable_rF5,
    oFFSET_StgRegTable_rF6,
    oFFSET_StgRegTable_rD1,
    oFFSET_StgRegTable_rD2,
    oFFSET_StgRegTable_rD3,
    oFFSET_StgRegTable_rD4,
    oFFSET_StgRegTable_rD5,
    oFFSET_StgRegTable_rD6,
    oFFSET_StgRegTable_rXMM1,
    oFFSET_StgRegTable_rXMM2,
    oFFSET_StgRegTable_rXMM3,
    oFFSET_StgRegTable_rXMM4,
    oFFSET_StgRegTable_rXMM5,
    oFFSET_StgRegTable_rXMM6,
    oFFSET_StgRegTable_rYMM1,
    oFFSET_StgRegTable_rYMM2,
    oFFSET_StgRegTable_rYMM3,
    oFFSET_StgRegTable_rYMM4,
    oFFSET_StgRegTable_rYMM5,
    oFFSET_StgRegTable_rYMM6,
    oFFSET_StgRegTable_rZMM1,
    oFFSET_StgRegTable_rZMM2,
    oFFSET_StgRegTable_rZMM3,
    oFFSET_StgRegTable_rZMM4,
    oFFSET_StgRegTable_rZMM5,
    oFFSET_StgRegTable_rZMM6,
    oFFSET_StgRegTable_rL1,
    oFFSET_StgRegTable_rSp,
    oFFSET_StgRegTable_rSpLim,
    oFFSET_StgRegTable_rHp,
    oFFSET_StgRegTable_rHpLim,
    oFFSET_StgRegTable_rCCCS,
    oFFSET_StgRegTable_rCurrentTSO,
    oFFSET_StgRegTable_rCurrentNursery,
    oFFSET_StgRegTable_rHpAlloc,
    oFFSET_stgEagerBlackholeInfo,
    oFFSET_stgGCEnter1,
    oFFSET_stgGCFun,
    oFFSET_Capability_r,
    oFFSET_bdescr_start,
    oFFSET_bdescr_free,
    oFFSET_bdescr_blocks,
    oFFSET_bdescr_flags,
    sIZEOF_CostCentreStack,
    oFFSET_CostCentreStack_mem_alloc,
    oFFSET_CostCentreStack_scc_count,
    oFFSET_StgHeader_ccs,
    oFFSET_StgHeader_ldvw,
    sIZEOF_StgSMPThunkHeader,
    oFFSET_StgEntCounter_allocs,
    oFFSET_StgEntCounter_allocd,
    oFFSET_StgEntCounter_registeredp,
    oFFSET_StgEntCounter_link,
    oFFSET_StgEntCounter_entry_count,
    sIZEOF_StgUpdateFrame_NoHdr,
    sIZEOF_StgMutArrPtrs_NoHdr,
    oFFSET_StgMutArrPtrs_ptrs,
    oFFSET_StgMutArrPtrs_size,
    sIZEOF_StgSmallMutArrPtrs_NoHdr,
    oFFSET_StgSmallMutArrPtrs_ptrs,
    sIZEOF_StgArrBytes_NoHdr,
    oFFSET_StgArrBytes_bytes,
    oFFSET_StgTSO_alloc_limit,
    oFFSET_StgTSO_cccs,
    oFFSET_StgTSO_stackobj,
    oFFSET_StgStack_sp,
    oFFSET_StgStack_stack,
    oFFSET_StgUpdateFrame_updatee,
    oFFSET_StgFunInfoExtraFwd_arity,
    sIZEOF_StgFunInfoExtraRev,
    oFFSET_StgFunInfoExtraRev_arity,
    mAX_SPEC_SELECTEE_SIZE,
    mAX_SPEC_AP_SIZE,
    mIN_PAYLOAD_SIZE,
    mIN_INTLIKE,
    mAX_INTLIKE,
    mIN_CHARLIKE,
    mAX_CHARLIKE,
    mUT_ARR_PTRS_CARD_BITS,
    mAX_Vanilla_REG,
    mAX_Float_REG,
    mAX_Double_REG,
    mAX_Long_REG,
    mAX_XMM_REG,
    mAX_Real_Vanilla_REG,
    mAX_Real_Float_REG,
    mAX_Real_Double_REG,
    mAX_Real_XMM_REG,
    mAX_Real_Long_REG,
    rESERVED_C_STACK_BYTES,
    rESERVED_STACK_WORDS,
    aP_STACK_SPLIM,
    wORD_SIZE,
    dOUBLE_SIZE,
    cINT_SIZE,
    cLONG_SIZE,
    cLONG_LONG_SIZE,
    bITMAP_BITS_SHIFT,
    tAG_BITS,
    wORDS_BIGENDIAN,
    dYNAMIC_BY_DEFAULT,
    lDV_SHIFT,
    iLDV_CREATE_MASK,
    iLDV_STATE_CREATE,
    iLDV_STATE_USE,
